""" package supplying optical glass data from vendor supplied data

    The ``opticalglass`` package currently supports the following vendors:

        - CDGM
        - Hoya
        - Ohara
        - Schott

    The spreadsheets from the vendors are in the ``data`` directory.
"""
